import ij.ImagePlus;
import ij.gui.GenericDialog;
import ij.measure.Calibration;
import ij.plugin.PlugIn;
import ij.process.ImageProcessor;
import imagescience.image.Aspects;
import imagescience.image.ColorImage;
import imagescience.image.Dimensions;
import imagescience.image.Image;
import imagescience.transform.Turn;
import imagescience.utility.I5DResource;
import java.awt.image.ColorModel;

public class TJ_Turn implements PlugIn {
    
    private static final String[] angles = {"0","90","180","270"};

    private static int zindex = 0;
    private static int yindex = 0;
    private static int xindex = 0;

    public void run(String arg) {

	if (!TJ.libcheck()) return;
	final ImagePlus imp = TJ.imageplus();
	if (imp == null) return;

	TJ.debug(TJ.name()+" "+TJ.version()+": Turn");

	GenericDialog gd = new GenericDialog(TJ.name()+": Turn");
	gd.addChoice("z-angle in degrees:",angles,angles[zindex]);
	gd.addChoice("y-angle in degrees:",angles,angles[yindex]);
	gd.addChoice("x-angle in degrees:",angles,angles[xindex]);
	gd.showDialog();

	if (gd.wasCanceled()) return;

	zindex = gd.getNextChoiceIndex();
	yindex = gd.getNextChoiceIndex();
	xindex = gd.getNextChoiceIndex();

	(new TJTurn()).run(imp,zindex,yindex,xindex);
    }

}

class TJTurn { // Separate class to make the plugin class independent of imagescience.jar
    
    void run(final ImagePlus imp,
	     final int zindex,
	     final int yindex,
	     final int xindex) {

	try {
	    final Image img = Image.wrap(imp);
	    final Turn turner = new Turn();
	    turner.debug(TJ_Options.debug);
	    final Image newimg = turner.run(img,zindex,yindex,xindex);
	    ImagePlus newimp = newimg.imageplus();
	    final Aspects newasps = newimg.aspects();
	    final Calibration cal = imp.getCalibration().copy();
	    cal.pixelWidth = newasps.x;
	    cal.pixelHeight = newasps.y;
	    cal.pixelDepth = newasps.z;
	    newimp.setCalibration(cal);
	    final ImageProcessor ip = imp.getProcessor();
	    final ImageProcessor newip = newimp.getProcessor();
	    final ColorModel cm = ip.getColorModel();
	    newip.setColorModel(cm);
	    newimp.getStack().setColorModel(cm);
	    final double[] minmax = newimg.extrema();
	    if (TJ_Options.adopt) newip.setMinAndMax(ip.getMin(),ip.getMax());
	    else newip.setMinAndMax(minmax[0],minmax[1]);
	    boolean i5dinstalled = false;
	    try { Class.forName("i5d.Image5D"); i5dinstalled = true; } catch (Exception e) { }
	    if (i5dinstalled && !(newimg instanceof ColorImage)) {
		final Dimensions newdims = newimg.dimensions();
		if (newdims.t > 1 || newdims.s > 1) {
		    newimp = I5DResource.convert(newimp,true);
		    if (I5DResource.instance(imp)) {
			I5DResource.transfer(imp,newimp);
			if (!TJ_Options.adopt)
			    I5DResource.minmax(newimp,minmax[0],minmax[1]);
		    }
		}
	    }
	    newimp.changes = TJ_Options.asksave;
	    newimp.show();

	} catch (OutOfMemoryError e) {
	    TJ.error("Not enough memory for this operation");

	} catch (UnknownError e) {
	    TJ.error("Could not create output image for some reason.\nPossibly there is not enough free memory");

	} catch (Throwable e) {
	    TJ.error("An unidentified error occurred while running the plugin");

	}
    }

}
